* INCLUDE FILE TO ESTIMATE R2-CHANGE F-TESTS 
  AND REGIONS OF SIGNIFICANCE FOR INTERACTIONS;

******************************************************************************;
/* To use R2Change Macro, enter:
	N=		Total Sample Size
	DataL=	Name of dataset that stores R2 for larger model	(on ODS)
	NpredL=	Number of predictors in larger model
	DataS=	Name of dataset that stores R2 for smaller model (on ODS)
	NpredS=	Number of predictors in smaller model */

* Macro to compute R2 change F-test;
%MACRO R2Change(N, DataL, NpredL, DataS, NpredS);
DATA _NULL_; SET &DataL.; CALL SYMPUT('R2L', Rsquare);	RUN;
DATA _NULL_; SET &DataS.; CALL SYMPUT('R2S', Rsquare);	RUN;
DATA Compare; 
	RsquareL=&R2L.; RsquareS=&R2S.; RsquareDiff = RsquareL - RsquareS; 
	NpredL=&NpredL.; NpredS=&NpredS.; 
	DFnum = NpredL-NpredS; DFdenom=&N.-NpredL-1;
	Fvalue = (RsquareDiff/DFnum)/((1-RsquareL)/DFdenom);
	Pvalue = 1-PROBF(Fvalue,DFnum,DFdenom);
RUN;
TITLE "R2 Change F-Test for &DataL. (L) vs. &DataS. (S)"; 
PROC PRINT DATA=Compare NOOBS; VAR RsquareL--Pvalue; RUN; TITLE;
%MEND R2Change;
******************************************************************************;

******************************************************************************;
/* To Use Asymp Macro, enter:
	DV=			Name of dependent variable
	AnoData=	Name of dataset that stores MSE for model (on ODS)
	InvData=	Name of dataset that stores X'X inv matrix for model (on ODS)
	Npred=		Number of predictors in model (include row of dots) */

* Macro to compute asymptotic covariance matrix;
%MACRO Asymp(DV, AnoData, InvData, Npred);
DATA _NULL_; SET &AnoData.; WHERE INDEX(Source,"Error")>0;
	CALL SYMPUT('MSE', MS); RUN;
DATA &InvData.; SET &InvData.; MSE=&MSE.;
	ARRAY list (%EVAL(2+&Npred.)) Intercept--&DV.;
	DO i=1 TO %EVAL(2+&Npred.); list(i)=MSE*list(i);
	END; DROP MSE i;
RUN;
/*TITLE "Asymptotic Covariance Matrix of Fixed Effects for &InvData."; 
PROC PRINT DATA=&InvData. NOOBS; RUN; TITLE;*/
%MEND Asymp;
******************************************************************************;

******************************************************************************;
/* To use Regions Macro, enter:
	FixData=	Name of dataset that stores fixed effects for model	(on ODS)
	InvData=	Name of dataset that stores X'X inv matrix for model  (on ODS)
	Pred=		Case-sensitive name of predictor effect regions are for 
	Mod=		Case-sensitive name of moderator effect (for region values)
	ModCenter=	Centering point of moderator predictor
	Interact=	Case-sensitive name of interaction effect */

%MACRO Regions(FixData, InvData, Pred, Mod, ModCenter, Interact);
DATA _NULL_; SET &FixData.; WHERE Parameter="&Pred.";
	CALL SYMPUT('Bpred', Estimate); CALL SYMPUT('SEpred', StdErr); RUN;	
DATA _NULL_; SET &FixData.; WHERE Parameter="&Interact.";
	CALL SYMPUT('Binter', Estimate); CALL SYMPUT('SEinter', StdErr); RUN; 
DATA _NULL_; SET &InvData.; WHERE INDEX(Parameter,"&Interact.")>0;
	CALL SYMPUT('CovPredInt', &Pred.); RUN;	
%PUT Bpred=&Bpred. SEpred=&SEpred. Binter=&Binter. SEinter=&SEinter. CovPredInt=&CovPredInt.;
DATA Regions;  * Intermediate calculations;
	A=(1.96*1.96)*(&SEinter.*&SEinter.)-(&Binter.*&Binter.);
	B=2*((1.96*1.96)*&CovPredInt.-(&Bpred.*&Binter.));
	C=(1.96*1.96*&SEpred.*&SEpred.)-(&Bpred.*&Bpred.);
	CenteredLower=((-1*B)+SQRT((B*B)-4*A*C))/(2*A); CALL SYMPUT('cenlower',ROUND(CenteredLower,.001));
	CenteredUpper=((-1*B)-SQRT((B*B)-4*A*C))/(2*A); CALL SYMPUT('cenupper',ROUND(CenteredUpper,.001));
	UncenteredLower=CenteredLower+&ModCenter.; CALL SYMPUT('uncenlower',ROUND(UncenteredLower,.001));
	UncenteredUpper=CenteredUpper+&ModCenter.; CALL SYMPUT('uncenupper',ROUND(UncenteredUpper,.001));
RUN;
TITLE1 "The effect of &pred. will be significant at centered values of &mod. BELOW the lower bound"; 
TITLE2 "and ABOVE the upper bound, which translate to these uncentered lower and upper bounds.";  
PROC PRINT DATA=Regions NOOBS; VAR CenteredLower--UncenteredUpper; RUN; TITLE1;	TITLE2;
%MEND Regions;
******************************************************************************;
